<?php
/**
 * PHPSense Pagination Class
 *
 * PHP tutorials and scripts
 *
 * @package		PHPSense
 * @author		Jatinder Singh Thind
 * @copyright	Copyright (c) 2006, Jatinder Singh Thind
 * @link		http://www.phpsense.com
 */

// ------------------------------------------------------------------------


class PS_Pagination {
	var $php_self;
	var $rows_per_page = 10; //Number of records to display per page
	var $total_rows = 0; //Total number of rows returned by the query
	var $links_per_page = 5; //Number of links to display per page
	var $append = ""; //Paremeters to append to pagination links
	var $sql = "";
	var $fields = "";
	var $debug = false;
	var $conn = false;
	var $page = 1;
	var $max_pages = 0;
	var $offset = 0;
	var $actionEvent = false;
	var $event = '';
	
	const PAGE_TAG = '::PAGE::';
	
	/**
	 * Constructor
	 *
	 * @param resource $connection Mysql connection link
	 * @param string $sql SQL query to paginate. Example : SELECT * FROM users
	 * @param integer $rows_per_page Number of records to display per page. Defaults to 10
	 * @param integer $links_per_page Number of links to display per page. Defaults to 5
	 * @param string $append Parameters to be appended to pagination links 
	 */
	
	function PS_Pagination($connection, $sql, $rows_per_page = 10, $links_per_page = 5, $append = "", $page = "")
	{
		$this->fields = (isset($sql['fields'])) ? $sql['fields'] : "*";
		
		$this->conn = $connection;
		$this->sqlCond = $sql;
        $this->sql = "SELECT " . $this->fields . " FROM ".$sql['table'];
		if (isset($sql['cond']) && $sql['cond'] != '')
			$this->sql .= ' WHERE ' . $sql['cond'];
		if (isset($sql['group']) && $sql['group'] != '')
			$this->sql .= ' GROUP BY ' . $sql['group'];
		if (isset($sql['order']) && $sql['order'] != '')
			$this->sql .= ' ORDER BY ' . $sql['order'];
		$this->rows_per_page = (int)$rows_per_page;
		if (intval($links_per_page ) > 0)
		{
			$this->links_per_page = (int)$links_per_page;
		}
		else
		{
			$this->links_per_page = 5;
		}
		$this->append = $append;
		$this->php_self = htmlspecialchars($_SERVER['PHP_SELF']);
		if ($page == "")
		{
			if (isset($_GET['page'] ))
			{
				$this->page = intval($_GET['page'] );
			}
		}
		else
		{
			$this->page = $page;
		}
	}
	
	/**
	 * Executes the SQL query and initializes internal variables
	 *
	 * @access public
	 * @return resource
	 */
	function paginate()
	{
		//Check for valid mysql connection
		if (! $this->conn || ! is_resource($this->conn ))
		{
			if ($this->debug)
				echo "MSSQL connection missing<br />";
			return false;
		}
		
		//Find total number of rows
		$all_rs = mysql_query($this->sql);
		if (! $all_rs)
		{
			if ($this->debug)
				echo "SQL query failed. Check your query.<br /><br />Error Returned: " . mysql_error();
			return false;
		}
		$this->total_rows = mysql_num_rows($all_rs );
		@mysql_close($all_rs );
		
		//Return FALSE if no rows found
		if ($this->total_rows == 0)
		{
			if ($this->debug)
				echo "Query returned zero rows.";
			return FALSE;
		}
		
		//Max number of pages
		$this->max_pages = ceil($this->total_rows / $this->rows_per_page );
		if ($this->links_per_page > $this->max_pages)
		{
			$this->links_per_page = $this->max_pages;
		}
		
		//Check the page value just in case someone is trying to input an aribitrary value
		if ($this->page > $this->max_pages || $this->page <= 0)
		{
			$this->page = 1;
		}
		
		//Calculate Offset
		$this->offset = $this->rows_per_page * ($this->page - 1);
		
		//Fetch the required result set
//		$this->sql = "SELECT TOP {$this->rows_per_page} *
//					  FROM ".$this->sqlCond['table']." WHERE ".$this->sqlCond['key']." NOT IN (SELECT TOP {$this->offset} ".$this->sqlCond['key'].
//					  " FROM ".$this->sqlCond['table']." WHERE ".$this->sqlCond['cond']." ) and ".$this->sqlCond['cond'];		
		$this->sql = "SELECT " . $this->fields . " FROM ".$this->sqlCond['table']." WHERE ".$this->sqlCond['cond']." LIMIT {$this->offset}, {$this->rows_per_page}";
		$this->sql = "SELECT " . $this->fields . " FROM ".$this->sqlCond['table'];
		if (isset($this->sqlCond['cond']) && $this->sqlCond['cond'] != '')
			$this->sql .= ' WHERE ' . $this->sqlCond['cond'];
		if (isset($this->sqlCond['group']) && $this->sqlCond['group'] != '')
			$this->sql .= ' GROUP BY ' . $this->sqlCond['group'];
		if (isset($this->sqlCond['order']) && $this->sqlCond['order'] != '')
			$this->sql .= ' ORDER BY ' . $this->sqlCond['order'];
		$this->sql .= " LIMIT {$this->offset}, {$this->rows_per_page}";
		
		$rs = @mysql_query($this->sql);
		if (! $rs)
		{
			if ($this->debug)
				echo "Pagination query failed. Check your query.<br /><br />Error Returned: " . mysql_error();
			return false;
		}
		return $rs;
	}
	
	/**
	 * Display the link to the first page
	 *
	 * @access public
	 * @param string $tag Text string to be displayed as the link. Defaults to 'First'
	 * @return string
	 */
	function renderFirst($tag = '<img src="images/first_icon.png">')
	{
		if ($this->total_rows == 0)
			return FALSE;
		
		if ($this->page == 1) {
			return '';
		} else {
			if ($this->actionEvent)
			{
				$event = str_replace(self::PAGE_TAG, 1, $this->event);
				return '<a href="#" onclick="'.$event.'">' . $tag . '</a>';
			}
			else
			{
				return '<a href="' . $this->php_self . '?page=1&' . $this->append . '">' . $tag . '</a>';
			}
		}
	}
	
	/**
	 * Display the link to the last page
	 *
	 * @access public
	 * @param string $tag Text string to be displayed as the link. Defaults to 'Last'
	 * @return string
	 */
	function renderLast($tag = '<img src="images/last_icon.png">')
	{
		if ($this->total_rows == 0)
			return FALSE;
		
		if ($this->page == $this->max_pages) {
			return '';
		} else {
			if ($this->actionEvent)
			{
				$event = str_replace(self::PAGE_TAG, $this->max_pages, $this->event);
				return '<a href="#" onclick="'.$event.'">' . $tag . '</a>';
			}
			else
			{
				return '<a href="' . $this->php_self . '?page=' . $this->max_pages . '&' . $this->append . '">' . $tag . '</a>';
			}
		}
	}
	
	/**
	 * Display the next link
	 *
	 * @access public
	 * @param string $tag Text string to be displayed as the link. Defaults to '>>'
	 * @return string
	 */
	function renderNext($tag = 'Next')
	{
		if ($this->total_rows == 0)
			return FALSE;
		
		if ($this->page < $this->max_pages) {
			if ($this->actionEvent)
			{
				$event = str_replace(self::PAGE_TAG, ($this->page + 1), $this->event);
				return '<a href="#" onclick="'.$event.'"><img src="images/next_icon.png"></a>';
			}
			else
			{
				return '<a href="' . $this->php_self . '?page=' . ($this->page + 1) . '&' . $this->append . '"><img src="images/next_icon.png"></a>';
			}
		} else {
			return "<!--<b><img src='images/next_icon.png'></b>-->";
		}
	}
	
	/**
	 * Display the previous link
	 *
	 * @access public
	 * @param string $tag Text string to be displayed as the link. Defaults to '<<'
	 * @return string
	 */
	function renderPrev($tag = 'Prev')
	{
		if ($this->total_rows == 0)
			return FALSE;
		
		if ($this->page > 1) {
			if ($this->actionEvent)
			{
				$event = str_replace(self::PAGE_TAG, ($this->page - 1), $this->event);
				return '<a href="#" onclick="'.$event.'"><img src="images/prev_icon.png"></a>';
			}
			else
			{
				return '<a href="' . $this->php_self . '?page=' . ($this->page - 1) . '&' . $this->append . '"><img src="images/prev_icon.png"></a>';
			}
			/*return ' <a href="' . $this->php_self . '?page=' . ($this->page - 1) . '&' . $this->append . '"><img src="images/pme-prev.png" border="0"></a>';*/
		} else {
			return  "<!--<b><img src='images/prev_icon.png'></b>-->";
		}
	}
	
	/**
	 * Display the page links
	 *
	 * @access public
	 * @return string
	 */
	function renderNav($prefix = '<span class="page_link">', $suffix = '</span>')
	{
		if ($this->total_rows == 0)
			return FALSE;
		
		$batch = ceil($this->page / $this->links_per_page );
		$end = $batch * $this->links_per_page;
		if ($end == $this->page) {
			//$end = $end + $this->links_per_page - 1;
		//$end = $end + ceil($this->links_per_page/2);
		}
		if ($end > $this->max_pages) {
			$end = $this->max_pages;
		}
		$start = $end - $this->links_per_page + 1;
		$links = '';
		
		for($i = $start; $i <= $end; $i ++) {
			if ($i == $this->page) {
				$links .= $prefix . '<b> ' . $i . ' </b>' . $suffix;
			} else {
				if ($this->actionEvent)
				{
					$event = str_replace(self::PAGE_TAG, $i, $this->event);
					$links .= '' . $prefix . '<a href="#" onclick="'.$event.'"> ' .$i. ' </a>'. $suffix;
				}
				else
				{
					$links .= '' . $prefix . '<a href="' . $this->php_self . '?page=' . $i . '&' . $this->append . '"> ' . $i . ' </a>' . $suffix . '';
				}
			}
		}
		
		return $links;
	}
	
	/**
	 * Display full pagination navigation
	 *
	 * @access public
	 * @return string
	 */
	function renderFullNav()
	{
		return $this->renderFirst() . '&nbsp;' . $this->renderPrev() . '&nbsp;' . $this->renderNav() . '&nbsp;' . $this->renderNext() . '&nbsp;' . $this->renderLast();
	}
	
	/**
	 * Set debug mode
	 *
	 * @access public
	 * @param bool $debug Set to TRUE to enable debug messages
	 * @return void
	 */
	function setDebug($debug)
	{
		$this->debug = $debug;
	}
	
	function setActionEvent($actionEvent, $event)
	{
		$this->actionEvent = $actionEvent;
		$this->event = $event;
	}
	
	function displayPageLinks()
	{
        return $this->renderPrev()." &nbsp;&nbsp;Page <b>".$this->page."</b> of <b>".$this->max_pages."</b>&nbsp;&nbsp; ".$this->renderNext();
	}
}
?>
